@@ -84,7 +84,7 @@ def __get_conn_data() -> dict[DbEngine, dict[DbParam, Any]]:
             default_setup = False
         else:
             prefix: str = str_positional(source=engine,
-                                         list_from=["mysql", "oracle", "postgres", "sqlserver"],
+                                         list_from=list(DbEngine),
                                          list_to=["MSQL", "ORCL", "PG", "SQLS"])
         result[engine] = {
             DbParam.NAME: env_get_str(key=f"{APP_PREFIX}_{prefix}_NAME"),
@@ -207,6 +207,7 @@ def _get_params(engine: DbEngine) -> dict[DbParam, Any]:
 
 
 def _except_msg(exception: Exception,
+                connection: Any | None,
                 engine: DbEngine) -> str:
     """
     Format and return the error message corresponding to the exception raised while accessing the database.
@@ -216,7 +217,8 @@ def _except_msg(exception: Exception,
     :return: the formatted error message
     """
     db_data: dict[DbParam, Any] = _DB_CONN_DATA.get(engine) or {}
-    return (f"Error accessing '{db_data.get(DbParam.NAME)}' "
+    conn_data: str = f", connection {id(connection)}," if connection else ""
+    return (f"Error accessing '{db_data.get(DbParam.NAME)}'{conn_data} "
             f"at '{db_data.get(DbParam.HOST)}': {str_sanitize(source=f'{exception}')}")
 
 
