@@ -3,7 +3,9 @@ from contextlib import suppress
 from datetime import datetime, UTC
 from enum import StrEnum, auto
 from logging import Logger
-from pypomes_core import APP_PREFIX, env_get_int, str_positional
+from pypomes_core import (
+    APP_PREFIX, env_get_int, str_positional, str_sanitize
+)
 from sys import getrefcount
 from time import sleep
 from threading import Lock
@@ -35,9 +37,9 @@ class _ConnStage(StrEnum):
     """
     Stage data of connections in pool.
     """
-    CONNECTION = auto()
-    TIMESTAMP = auto()
-    AVAILABLE = auto()
+    CONNECTION = auto()                 # the connection object
+    TIMESTAMP = auto()                  # creation datetime (POSIX timestamp)
+    AVAILABLE = auto()                  # status (True: available; False: taken)
 
 
 # available DbConnectionPool instances:
@@ -56,7 +58,7 @@ def __get_pool_params() -> dict[DbEngine, dict[_PoolParam, Any]]:
     Establish the connection pool parameters for supported databases, from values in environment variables.
 
     To specify database connection parameters with environment variables, use the set:
-      - *<APP_PREFIX>_DB_POOL_SIZE*: maximum number of connections in the pool (defaults to 20 connections)
+      - *<APP_PREFIX>_DB_POOL_SIZE*: maximum number of connections in the pool (defaults to 40 connections)
       - *<APP_PREFIX>_DB_POOL_TIMEOUT*: number of seconds to wait for a connection to become available,
         before failing the request (defaults to 60 seconds)
       - *<APP_PREFIX>_DB_POOL_RECYCLE*:  number of seconds after which connections in the pool are closed
@@ -73,13 +75,13 @@ def __get_pool_params() -> dict[DbEngine, dict[_PoolParam, Any]]:
 
     for engine in DbEngine:
         prefix: str = str_positional(source=engine,
-                                     list_from=["mysql", "oracle", "postgres", "sqlserver"],
+                                     list_from=list(DbEngine),
                                      list_to=["MSQL", "ORCL", "PG", "SQLS"])
         # establish pool size
         size: int = env_get_int(key=f"{APP_PREFIX}_{prefix}_POOL_SIZE")
-        if not size:
+        if not isinstance(size, int) or size < 0:
             size = env_get_int(key=f"{APP_PREFIX}_DB_POOL_SIZE",
-                               def_value=20)
+                               def_value=40)
         # establish pool timeout
         timeout: int = env_get_int(key=f"{APP_PREFIX}_{prefix}_POOL_TIMEOUT")
         if not timeout:
@@ -155,9 +157,9 @@ class DbConnectionPool:
         for pool configuration parameters specified in the environment variables suffixed with their
         uppercase names (nsmely, *POOL_SIZE*, *POOL_TIMEOUT*, and *POOL_RECYCLE*) are used.
         If still not specified, these are the default values used:
-          - *pool_size*: 20 connections
+          - *pool_size*: 40 connections
           - *pool_timeout*: 60 seconds
-          - *pool_recycle*: 3600 seconds (1 hour)
+          - *pool_recycle*: 3600 seconds (60 minutos)
 
         All instance attributes are final and should not be directly changed, lest the pool malfunction.
 
@@ -177,41 +179,50 @@ class DbConnectionPool:
         if engine:
             with _instances_lock:
                 if engine in _POOL_INSTANCES:
-                    msg = f"Attempt to create connection pool failed: {engine} pool already exists"
+                    msg: str = f"Attempt to create connection pool failed: {engine} pool already exists"
                     if logger:
-                        logger.warning(msg)
+                        logger.error(msg)
                     if isinstance(errors, list):
                         errors.append(msg)
                 else:
                     pool_params = _POOL_PARAMS[engine]
-                    # register this instance
-                    _POOL_INSTANCES[engine] = self
 
         if pool_params:
-            self.db_engine: Final[DbEngine] = engine
-            self.pool_size: Final[int] = pool_size or pool_params[_PoolParam.SIZE]
-            self.pool_timeout: Final[int] = pool_timeout or pool_params[_PoolParam.TIMEOUT]
-            self.pool_recycle: Final[int] = pool_recycle or pool_params[_PoolParam.RECYCLE]
-            self.stage_lock: Final[Lock] = Lock()
-            self.event_lock: Final[Lock] = Lock()
+            if pool_size is None:
+                pool_size = pool_params[_PoolParam.SIZE]
+            if pool_size > 0:
+                self.db_engine: Final[DbEngine] = engine
+                self.pool_size: Final[int] = pool_size
+                self.pool_timeout: Final[int] = pool_timeout or pool_params[_PoolParam.TIMEOUT]
+                self.pool_recycle: Final[int] = pool_recycle or pool_params[_PoolParam.RECYCLE]
+                self.stage_lock: Final[Lock] = Lock()
+                self.event_lock: Final[Lock] = Lock()
 
-            self.event_callbacks: Final[dict[DbPoolEvent, callable]] = {
-                DbPoolEvent.CREATE: None,
-                DbPoolEvent.CHECKOUT: None,
-                DbPoolEvent.CHECKIN: None,
-                DbPoolEvent.CLOSE: None
-            }
-            self.event_stmts: Final[dict[DbPoolEvent, list[str]]] = {
-                DbPoolEvent.CREATE: [],
-                DbPoolEvent.CHECKOUT: [],
-                DbPoolEvent.CHECKIN: [],
-                DbPoolEvent.CLOSE: []
-            }
-            self.conn_data: Final[list[dict[_ConnStage, Any]]] = []
+                self.event_callbacks: Final[dict[DbPoolEvent, callable]] = {
+                    DbPoolEvent.CREATE: None,
+                    DbPoolEvent.CHECKOUT: None,
+                    DbPoolEvent.CHECKIN: None,
+                    DbPoolEvent.CLOSE: None
+                }
+                self.event_stmts: Final[dict[DbPoolEvent, list[str]]] = {
+                    DbPoolEvent.CREATE: [],
+                    DbPoolEvent.CHECKOUT: [],
+                    DbPoolEvent.CHECKIN: [],
+                    DbPoolEvent.CLOSE: []
+                }
+                self.conn_data: Final[list[dict[_ConnStage, Any]]] = []
 
-            if logger:
-                logger.debug(msg=f"{self.db_engine} pool created: size {self.pool_size}, "
-                                 f"timeout {self.pool_timeout}, recycle {self.pool_recycle}")
+                # register this instance
+                _POOL_INSTANCES[engine] = self
+                if logger:
+                    logger.info(msg=f"{self.db_engine} pool created: size {self.pool_size}, "
+                                    f"timeout {self.pool_timeout}, recycle {self.pool_recycle}")
+            else:
+                msg: str = f"{engine} pool not created: specified size was {pool_size}"
+                if logger:
+                    logger.error(msg=msg)
+                    if isinstance(errors, list):
+                        errors.append(msg)
 
     def connect(self,
                 errors: list[str] = None,
@@ -223,21 +234,90 @@ class DbConnectionPool:
         :param logger: optional logger
         :return: a connection from the pool, or *None* if error
         """
-        # obtain a pooled connection
+        # initialize the return variable
         result: Any = None
 
+        # obtain a pooled connection
         if not isinstance(errors, list):
             errors = []
         start: float = datetime.now(tz=UTC).timestamp()
         while not result:
             with self.stage_lock:
-                self.__revise(logger=logger,
-                              errors=errors)
-                for conn_item in self.conn_data:
-                    if conn_item[_ConnStage.AVAILABLE]:
-                        conn_item[_ConnStage.AVAILABLE] = False
-                        result = conn_item[_ConnStage.CONNECTION]
-                        break
+                # traverse the connection data in reverse order
+                for i in range(len(self.conn_data) - 1, -1, -1):
+                    conn: Any = self.conn_data[i][_ConnStage.CONNECTION]
+
+                    # connection is available
+                    if self.conn_data[i][_ConnStage.AVAILABLE]:
+                        # connection exhausted its lifetime
+                        if datetime.now(tz=UTC).timestamp() > \
+                                self.conn_data[i][_ConnStage.TIMESTAMP] + self.pool_recycle:
+                            # close the exhausted connection
+                            self.__act_on_event(event=DbPoolEvent.CLOSE,
+                                                conn=conn,
+                                                errors=errors,
+                                                logger=logger)
+                            with suppress(Exception):
+                                conn.close()
+                            # dispose of exhausted connection
+                            self.conn_data.pop(i)
+                            if logger:
+                                logger.debug(msg=f"The exhausted connection {id(conn)} was removed from the "
+                                                 f"{self.db_engine} pool, count = {len(self.conn_data)}")
+                        # connection is candidate
+                        else:
+                            # assert connection health
+                            stmt = "SELECT 1"
+                            if self.db_engine == DbEngine.ORACLE:
+                                stmt += " FROM DUAL"
+                            try:
+                                cursor: Any = conn.cursor()
+                                # parameter name:
+                                #   - oracledb (oracle):'statement'
+                                #   - psycopg2 (postgres): 'query'
+                                #   - pyodbc (sqlserver): 'sql'
+                                #   - mysql-connector-python: none
+                                cursor.execute(stmt)
+                                cursor.fetchone()
+                                cursor.close()
+                                # mark the connection as taken
+                                self.conn_data[i][_ConnStage.AVAILABLE] = False
+                                result = conn
+                                break
+                            except Exception as e:
+                                # dispose of bad connection
+                                self.conn_data.pop(i)
+                                if logger:
+                                    logger.warning(f"Connection {id(conn)} "
+                                                   f"assessment failed: {str_sanitize(source=f'{e}')}")
+                                    logger.debug(msg=f"The bad connection {id(conn)} was removed from the "
+                                                     f"{self.db_engine} pool, count = {len(self.conn_data)}")
+
+                    # connection was closed elsewhere
+                    elif hasattr(conn, "closed") and conn.closed:
+                        # dispose of closed connection
+                        self.conn_data.pop(i)
+                        if logger:
+                            logger.debug(msg=f"The closed connection {id(conn)} was removed from the "
+                                             f"{self.db_engine} pool, count = {len(self.conn_data)}")
+
+                    # connection is no longer in use - these are the 3 remaining references:
+                    #   - pool repository 'self.conn_data[i][_ConnStage.CONNECTION]'
+                    #   - local variable 'conn'
+                    #   - argument to 'getrefcount()'
+                    elif getrefcount(conn) < 4:
+                        # reclaim the connection
+                        self.__act_on_event(event=DbPoolEvent.CHECKIN,
+                                            conn=conn,
+                                            errors=errors,
+                                            logger=logger)
+                        if not errors:
+                            # mark the connection as available
+                            self.conn_data[i][_ConnStage.AVAILABLE] = True
+                            if logger:
+                                logger.debug(msg=f"The stray connection {id(conn)} "
+                                                 f"was reclaimed by the {self.db_engine} pool")
+
                 if not result and len(self.conn_data) < self.pool_size:
                     match self.db_engine:
                         case DbEngine.MYSQL:
@@ -260,7 +340,7 @@ class DbConnectionPool:
                             result = sqlserver_pomes.connect(autocommit=False,
                                                              errors=errors,
                                                              logger=logger)
-                    if result:
+                    if not errors:
                         self.__act_on_event(event=DbPoolEvent.CREATE,
                                             conn=result,
                                             errors=errors,
@@ -268,33 +348,30 @@ class DbConnectionPool:
                         if not errors:
                             # store the connection
                             self.conn_data.append({
-                                _ConnStage.TIMESTAMP: datetime.now(tz=UTC).timestamp(),
                                 _ConnStage.AVAILABLE: False,
-                                _ConnStage.CONNECTION: result
+                                _ConnStage.CONNECTION: result,
+                                _ConnStage.TIMESTAMP: datetime.now(tz=UTC).timestamp()
                             })
                             if logger:
-                                logger.debug(msg=f"Connection {id(result)} "
-                                                 f"created by the {self.db_engine} pool")
+                                logger.debug(msg=f"Connection {id(result)} created by the "
+                                                 f"{self.db_engine} pool, count = {len(self.conn_data)}")
             if not result:
-                if datetime.now(tz=UTC).timestamp() - start < self.pool_timeout:
-                    sleep(seconds=1.5)
+                if self.pool_timeout > datetime.now(tz=UTC).timestamp() - start:
+                    sleep(1.5)
                 else:
                     errors.append("Timeout waiting for available connection")
                     break
-
-        if result and not errors:
+        if result:
             self.__act_on_event(event=DbPoolEvent.CHECKOUT,
                                 conn=result,
                                 errors=errors,
                                 logger=logger)
-            if not errors and logger:
+            if errors:
+                # a connection was obtained, but errors prevent its usage
+                result = None
+            elif logger:
                 logger.debug(msg=f"Connection {id(result)} "
-                                 f"retrieved from the {self.db_engine} pool")
-
-        # a connection may have been found, but errors would prevent it from being used
-        if errors:
-            result = None
-
+                                 f"delivered by the {self.db_engine} pool")
         return result
 
     def on_event_actions(self,
@@ -359,6 +436,7 @@ class DbConnectionPool:
                                         errors=errors,
                                         logger=logger)
                     if not errors:
+                        # mark the connection as available
                         data[_ConnStage.AVAILABLE] = True
                         result = True
                         if logger:
@@ -423,7 +501,7 @@ class DbConnectionPool:
                          errors=errors,
                          logger=logger)
                 if logger:
-                    logger.debug(msg=f"Event '{event}' on connection {id(conn)}: "
+                    logger.debug(msg=f"Event '{event}' on {self.db_engine} connection {id(conn)}: "
                                      f"'{callback.__name__}' invoked, {len(errors)} errors")
             # execute the statements
             if not errors and self.event_stmts[event]:
@@ -435,57 +513,11 @@ class DbConnectionPool:
                                         errors=errors,
                                         logger=logger)
                     if logger:
-                        logger.debug(msg=f"Event '{event}' on connection {id(conn)}: "
+                        logger.debug(msg=f"Event '{event}' on {self.db_engine} connection {id(conn)}: "
                                          f"'{stmt}' executed, {len(errors)} errors")
                     if errors:
                         break
 
-    def __revise(self,
-                 errors: list[str] = None,
-                 logger: Logger = None) -> None:
-        """
-        Revise the pool, reclaiming or disposing of connections, if applicable.
-
-        This operation must be invoked within the guardrails of *self.stage_lock*.
-        """
-        if not isinstance(errors, list):
-            errors = []
-        # traverse the connection data in reverse order
-        length: int = len(self.conn_data)
-        rev_data: list[dict[_ConnStage, Any]] = list(reversed(self.conn_data))
-        for inx, data in enumerate(rev_data):
-            # connection was closed elsewhere
-            if hasattr(data[_ConnStage.CONNECTION], "closed") and data[_ConnStage.CONNECTION].closed:
-                # dispose of closed connection
-                self.conn_data.pop(length - inx - 1)
-            elif data[_ConnStage.AVAILABLE]:
-                # connect exausted its lifetime
-                if datetime.now(tz=UTC).timestamp() > data[_ConnStage.TIMESTAMP] + self.pool_recycle:
-                    # close the connection
-                    self.__act_on_event(event=DbPoolEvent.CLOSE,
-                                        conn=data[_ConnStage.CONNECTION],
-                                        errors=errors,
-                                        logger=logger)
-                    with suppress(Exception):
-                        data[_ConnStage.CONNECTION].close()
-                    # dispose of closed connection
-                    self.conn_data.pop(length - inx - 1)
-                    if logger:
-                        logger.debug(msg=f"Connection {id(data[_ConnStage.CONNECTION])} "
-                                         f"closed by the {self.db_engine} pool")
-            # with only 2 references (1 held here, and 1 held by the pool), connection is no longer in use
-            elif getrefcount(data[_ConnStage.CONNECTION]) < 3:
-                # reclaim the connection
-                self.__act_on_event(event=DbPoolEvent.CHECKIN,
-                                    conn=data[_ConnStage.CONNECTION],
-                                    errors=errors,
-                                    logger=logger)
-                if not errors:
-                    data[_ConnStage.AVAILABLE] = True
-                    if logger:
-                        logger.debug(msg=f"Connection {id(data[_ConnStage.CONNECTION])} "
-                                         f"reclaimed by the {self.db_engine} pool")
-
 
 def db_get_pool(engine: DbEngine = None) -> DbConnectionPool | None:
     """
